@@ -321,6 +321,12 @@ class IgorSqliteDB:
         # if strGene == 'D':
         #    print(sqlSelect)
         records = cur.fetchall()
+
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnIgorIndexedSeq)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
+
         with open(flnIgorIndexedSeq, "w") as ofile:
             ofile.write("seq_index" + sep + "sequence" + "\n")
             for record in records:
@@ -406,6 +412,10 @@ class IgorSqliteDB:
         cur.execute(sqlSelect)
         records = cur.fetchall()
         # str_file_header = "seq_index" + sep + "sequence" + "\n"
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnGeneTemplate)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
         with open(flnGeneTemplate, "w") as ofile:
             # ofile.write(str_file_header)
             for record in records:
@@ -502,13 +512,20 @@ class IgorSqliteDB:
         cur = self.conn.cursor()
         cur.execute(sqlSelect)
         records = cur.fetchall()
+
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnGeneAnchors)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
+
         str_file_header = "gene;anchor_index;function\n"
         with open(flnGeneAnchors, "w") as ofile:
             ofile.write(str_file_header)
             for record in records:
-                ofile.write(str(record[0]) + sep)
-                ofile.write(str(record[1]) + sep)
-                ofile.write(str(record[2]) + '\n')
+                if not record[1] is None:
+                    ofile.write(str(record[0]) + sep)
+                    ofile.write(str(record[1]) + sep)
+                    ofile.write(str(record[2]) + '\n')
 
     def delete_IgorGeneAnchors_Tables(self):
         """
@@ -797,6 +814,11 @@ class IgorSqliteDB:
         cur.execute(sqlSelect)
         records = cur.fetchall()
 
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnGeneTemplate)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
+
         str_file_header = "seq_index"+sep+"gene_name"+sep+"score"+sep+"offset"+sep+"insertions"+sep+"deletions"+sep+"mismatches"+sep+"length"+sep+"5_p_align_offset"+sep+"3_p_align_offset" + "\n"
         with open(flnGeneTemplate, "w") as ofile:
             ofile.write(str_file_header)
@@ -889,6 +911,11 @@ class IgorSqliteDB:
         #    print(sqlSelect)
         records = cur.fetchall()
 
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnIgorIndexedCDR3)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
+
         str_file_header = "seq_index;v_anchor;j_anchor;CDR3;CDR3_aa" + "\n"
         with open(flnIgorIndexedCDR3, "w") as ofile:
             ofile.write(str_file_header)
@@ -1303,6 +1330,11 @@ class IgorSqliteDB:
         cur.execute(sqlSelect)
         records = cur.fetchall()
 
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnIgorBestScenarios)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
+
         # str_file_header = "seq_index;gene_name;score;offset;insertions;deletions;mismatches;length;5_p_align_offset;3_p_align_offset" + "\n"
         with open(flnIgorBestScenarios, "w") as ofile:
             ofile.write(str_file_header)
@@ -1324,27 +1356,53 @@ class IgorSqliteDB:
         from .AIRR import AIRR_VDJ_rearrangement
         import airr
 
-        # FIXME: IF VDJ THEN:
-        airr_fields = AIRR_VDJ_rearrangement.list_of_fields()
-        airr_rearrangement_writer = airr.create_rearrangement(flnAIRR_arrangement, fields=airr_fields)
-        for seq_index, sequence in self.fetch_IgorIndexedSeq_records():
-            scenarios_list = self.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)
-            for scenario in scenarios_list:
-                v_best_aln = self.get_best_IgorAlignment_data_By_seq_index("V", seq_index)
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnAIRR_arrangement)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
 
-                airr_rearrangement_dict = mdl.get_AIRR_VDJ_rearragement_dict_from_scenario(scenario, sequence, v_offset=v_best_aln.offset)
-                airr_rearrangement_dict['scenario_rank'] = scenario.scenario_rank
-                airr_rearrangement_dict['scenario_proba_cond_seq'] = scenario.scenario_proba_cond_seq
-                airr_rearrangement_dict['pgen'] = self.fetch_IgorPgen_By_seq_index(seq_index)[1]
-                cdr3_record = self.fetch_IgorIndexedCDR3_By_seq_index(seq_index)
-                airr_rearrangement_dict['junction'] = cdr3_record[3]
-                airr_rearrangement_dict['junction_aa'] = cdr3_record[4]
-                airr_rearrangement_writer.write(airr_rearrangement_dict)
+        # IF VDJ THEN:
+        b_D_gene = ( len([ event.event_type  for event in mdl.parms.Event_list if event.nickname == 'd_gene']) > 0 )
 
-        airr_rearrangement_writer.close()
+        if b_D_gene:
+            airr_fields = AIRR_VDJ_rearrangement.list_of_fields()
+            airr_rearrangement_writer = airr.create_rearrangement(flnAIRR_arrangement, fields=airr_fields)
+            for seq_index, sequence in self.fetch_IgorIndexedSeq_records():
+                scenarios_list = self.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)
+                for scenario in scenarios_list:
+                    v_best_aln = self.get_best_IgorAlignment_data_By_seq_index("V", seq_index)
 
+                    airr_rearrangement_dict = mdl.get_AIRR_VDJ_rearragement_dict_from_scenario(scenario, sequence, v_offset=v_best_aln.offset)
+                    airr_rearrangement_dict['scenario_rank'] = scenario.scenario_rank
+                    airr_rearrangement_dict['scenario_proba_cond_seq'] = scenario.scenario_proba_cond_seq
+                    airr_rearrangement_dict['pgen'] = self.fetch_IgorPgen_By_seq_index(seq_index)[1]
+                    cdr3_record = self.fetch_IgorIndexedCDR3_By_seq_index(seq_index)
+                    airr_rearrangement_dict['junction'] = cdr3_record[3]
+                    airr_rearrangement_dict['junction_aa'] = cdr3_record[4]
+                    airr_rearrangement_writer.write(airr_rearrangement_dict)
 
+            airr_rearrangement_writer.close()
 
+        else:
+            # FIXME: IF VJ THEN:
+            airr_fields = AIRR_VDJ_rearrangement.list_of_fields()
+            airr_rearrangement_writer = airr.create_rearrangement(flnAIRR_arrangement, fields=airr_fields)
+            for seq_index, sequence in self.fetch_IgorIndexedSeq_records():
+                scenarios_list = self.get_IgorBestScenarios_By_seq_index_IgorModel(seq_index, mdl)
+                for scenario in scenarios_list:
+                    v_best_aln = self.get_best_IgorAlignment_data_By_seq_index("V", seq_index)
+
+                    airr_rearrangement_dict = mdl.get_AIRR_VJ_rearragement_dict_from_scenario(scenario, sequence,
+                                                                                               v_offset=v_best_aln.offset)
+                    airr_rearrangement_dict['scenario_rank'] = scenario.scenario_rank
+                    airr_rearrangement_dict['scenario_proba_cond_seq'] = scenario.scenario_proba_cond_seq
+                    airr_rearrangement_dict['pgen'] = self.fetch_IgorPgen_By_seq_index(seq_index)[1]
+                    cdr3_record = self.fetch_IgorIndexedCDR3_By_seq_index(seq_index)
+                    airr_rearrangement_dict['junction'] = cdr3_record[3]
+                    airr_rearrangement_dict['junction_aa'] = cdr3_record[4]
+                    airr_rearrangement_writer.write(airr_rearrangement_dict)
+
+            airr_rearrangement_writer.close()
 
         # n_d_5_del = self.mdlParms.Event_dict[strEv].loc[self.id_d_5_del]['value']
         # name_D = self.mdlParms.Event_dict[strEv].loc[self.id_d_gene]['name']
@@ -1443,6 +1501,10 @@ class IgorSqliteDB:
 
     def write_IgorPgen_to_CSV(self, flnIgorPgen, sep=';'):
         print("Writing Pgen to file: ", flnIgorPgen)
+        import pathlib
+        import os
+        directory_name = os.path.dirname(flnIgorPgen)
+        pathlib.Path(directory_name).mkdir(parents=True, exist_ok=True)
         str_header = "seq_index"+sep+"Pgen_estimate\n"
         with open(flnIgorPgen, "w") as ofile:
             # seq_index,Pgen_estimate
@@ -1463,7 +1525,9 @@ class IgorSqliteDB:
     ###### return IGoR Model
     def get_IgorModel(self):
         from .IgorIO import IgorModel
-        mdl = IgorModel.load_from_parms_marginals_object( self.get_IgorModel_Parms(), self.get_IgorModel_Marginals() )
+        mdl_parms = self.get_IgorModel_Parms()
+        mdl_marginals = self.get_IgorModel_Marginals()
+        mdl = IgorModel.load_from_parms_marginals_object( mdl_parms, mdl_marginals )
         return mdl
 
     def get_IgorModel_Marginals(self):
@@ -1488,7 +1552,7 @@ class IgorSqliteDB:
             records = self.execute_select_query("SELECT * FROM IgorMM_"+event_nickname+";")
 
             print(event_nickname) # , records)
-            events_tmp = [x[3:] for x in col_names if x != 'P']
+            events_tmp = [x[3:] for x in col_names if x != 'P'] # remove the 3 characters 'id_'
             marginals.network_dict[event_nickname] = events_tmp
             df = pd.DataFrame(np.array(records), columns=events_tmp+['P'])
             dimensions = list()
@@ -1496,7 +1560,28 @@ class IgorSqliteDB:
                 dim_ev = len(df[event_name].unique())
                 dimensions.append(dim_ev)
 
-            marginals.marginals_dict[event_nickname] = df['P'].to_numpy().reshape(dimensions)
+            np_marginals = df['P'].to_numpy().reshape(dimensions)
+
+            # FIXME TRANSPOSE WITH DIMENSIONS
+            axes_for_transpose = list()
+            aux_axes_for_transpose = list()
+
+            new_dict_list = list()
+            aux_new_dict_list = list()
+            for ii, str_event in enumerate(events_tmp):
+                if not str_event == event_nickname:
+                    axes_for_transpose.append(ii)
+                    new_dict_list.append(str_event)
+                else:
+                    aux_axes_for_transpose.append(ii)
+                    aux_new_dict_list.append(str_event)
+
+            marginals.network_dict[event_nickname] = new_dict_list + aux_new_dict_list
+            marginals.marginals_dict[event_nickname] = np.transpose(np_marginals, tuple(axes_for_transpose+aux_axes_for_transpose))
+
+            # FIXME:
+            # 1. GEt position of event_nickname in list marginals.network_dict[event_nickname]
+            # and put it at the end. marginals.network_dict[event_nickname]
 
         return marginals
 
@@ -1817,6 +1902,12 @@ class IgorSqliteDB:
 
         return flag_db
 
+    def Q_CDR3_Anchors_in_db(self, strGene):
+        flag_table = False
+        tables_list = self.get_list_of_tables_with_name('Igor' + strGene + 'GeneCDR3Anchors')
+        if len(tables_list) == 1: flag_table = True
+        return flag_table
+
     def Q_ref_genome_in_db_by_gene(self, strGene):
         flag_table = False
         tables_list = self.get_list_of_tables_with_name('Igor'+strGene+'GeneTemplate')
